require "spec_helper"

describe Bindep::OS do
  HAVE_LIST = {
    "brew?" => "brew",  
    "apt?" => "apt-get",  
    "yum?" => "yum",  
    "npm?" => "npm",
    "rubygem?" => "gem",
  }

  INSTALL_LIST = {
    "brew_install" => "brew install pkg",  
    "apt_install" => "apt-get install -y pkg",  
    "yum_install" => "yum install -y pkg",  
    "npm_install" => "npm install -g pkg",
    "rubygem_install" => "gem install pkg",
  }

  before :each do
    Bindep::OS.instance_variable_set(:@command_exists_cache, {})
  end

  HAVE_LIST.each_pair do |method, command|
    it "has a #{method} method which caches result" do
      expect(Bindep::Helpers).to receive(:command_exists?).with(command).once.and_return(false)

      2.times { Bindep::OS.send method.to_sym }
    end
  end

  [ true, false ].each do |sudo|
    sudo_string = sudo ? "sudo" : ""

    INSTALL_LIST.each_pair do |method, command|
      it "has a corrent #{method} method with sudo #{sudo}" do
        expect(Bindep::OS).to receive(:sudo).and_return(sudo_string)

        expect(Bindep::OS.send(method.to_sym, "pkg")).to eq "#{sudo_string} #{command}".strip
      end
    end
  end

  it "requires sudo if no brew is available (e.g. not on mac os)" do
    expect(Bindep::OS).to receive(:brew?).and_return(false)

    expect(Bindep::OS.sudo).to eq "sudo"
  end

  it "requires no sudo if brew is available" do
    expect(Bindep::OS).to receive(:brew?).and_return(true)

    expect(Bindep::OS.sudo).to eq ""
  end
end
